/* GemRB - Infinity Engine Emulator
 * Copyright (C) 2003 The GemRB Project
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * $Id: Label.h 6005 2009-05-19 10:21:38Z lynxlupodian $
 *
 */

/**
 * @file Label.h
 * Declares Label widget for displaying static texts
 * @author GemRB Developement Team
 */

#ifndef LABEL_H
#define LABEL_H

#include "Control.h"
#include "Font.h"
#include "../../includes/RGBAColor.h"

class Palette;

// !!! Keep these synchronized with GUIDefines.py !!!
#define IE_GUI_LABEL_ON_PRESS      0x06000000

#ifdef WIN32

#ifdef GEM_BUILD_DLL
#define GEM_EXPORT __declspec(dllexport)
#else
#define GEM_EXPORT __declspec(dllimport)
#endif

#else
#define GEM_EXPORT
#endif

/**
 * @class Label
 * Label widget for displaying static texts in the GUI
 */

class GEM_EXPORT Label : public Control {
public: 
	Label(Font* font);
	~Label();
	/** Draws the Control on the Output Display */
	void Draw(unsigned short x, unsigned short y);
	/** This function sets the actual Label Text */
	int SetText(const char* string, int pos = 0);
	/** Sets the Foreground Font Color */
	void SetColor(Color col, Color bac);
	/** Sets the Alignment of Text */
	void SetAlignment(unsigned char Alignment);
	/** Simply returns the pointer to the text, don't modify it! */
	const char* QueryText();

	/** Mouse Button Down */
	void OnMouseUp(unsigned short x, unsigned short y, unsigned short Button,
		unsigned short Mod);
	/** Set handler for specified event */
	bool SetEvent(int eventType, const char *handler);
	/** Use the RGB Color for the Font */
	bool useRGB;
	/** OnPress Scripted Event Function Name */
	EventHandler LabelOnPress;
private: // Private attributes
	/** Text String Buffer */
	char* Buffer;
	/** Font for Text Writing */
	Font* font;
	/** Foreground & Background Colors */
	Palette* palette;

	/** Alignment Variable */
	unsigned char Alignment;
};

#endif
